@@ -7,7 +7,7 @@ use twilight_model::{
     channel::{
         embed::Embed,
         message::{AllowedMentions, MessageFlags},
-        Message,
+        Attachment, Message,
     },
     id::{ChannelId, MessageId},
 };
@@ -80,6 +80,8 @@ pub enum UpdateMessageErrorType {
 struct UpdateMessageFields {
     #[serde(skip_serializing_if = "Option::is_none")]
     pub(crate) allowed_mentions: Option<AllowedMentions>,
+    #[serde(skip_serializing_if = "Vec::is_empty")]
+    pub attachments: Vec<Attachment>,
     // We don't serialize if this is Option::None, to avoid overwriting the
     // field without meaning to.
     //
@@ -155,6 +157,28 @@ impl<'a> UpdateMessage<'a> {
         }
     }
 
+    /// Specify an attachment already present in the target message to keep.
+    ///
+    /// If called, all unspecified attachments will be removed from the message.
+    /// If not called, all attachments will be kept.
+    pub fn attachment(mut self, attachment: Attachment) -> Self {
+        self.fields.attachments.push(attachment);
+
+        self
+    }
+
+    /// Specify multiple attachments already present in the target message to keep.
+    ///
+    /// If called, all unspecified attachments will be removed from the message.
+    /// If not called, all attachments will be kept.
+    pub fn attachments(mut self, attachments: impl IntoIterator<Item = Attachment>) -> Self {
+        self.fields
+            .attachments
+            .extend(attachments.into_iter().collect::<Vec<Attachment>>());
+
+        self
+    }
+
     /// Set the content of the message.
     ///
     /// Pass `None` if you want to remove the message content.
