@@ -3,7 +3,7 @@
 use crate::{
     client::Client,
     error::{Error as HttpError, Result},
-    request::{self, validate, AuditLogReason, AuditLogReasonError, Pending, Request},
+    request::{self, validate, AuditLogReason, AuditLogReasonError, Form, Pending, Request},
     routing::Route,
 };
 use serde::Serialize;
@@ -12,7 +12,7 @@ use std::{
     fmt::{Display, Formatter, Result as FmtResult},
 };
 use twilight_model::{
-    channel::{embed::Embed, message::AllowedMentions},
+    channel::{embed::Embed, message::AllowedMentions, Attachment},
     id::{MessageId, WebhookId},
 };
 
@@ -106,12 +106,16 @@ pub enum UpdateWebhookMessageErrorType {
 struct UpdateWebhookMessageFields {
     #[serde(skip_serializing_if = "Option::is_none")]
     allowed_mentions: Option<AllowedMentions>,
+    #[serde(skip_serializing_if = "Vec::is_empty")]
+    attachments: Vec<Attachment>,
     #[allow(clippy::option_option)]
     #[serde(skip_serializing_if = "Option::is_none")]
     content: Option<Option<String>>,
     #[allow(clippy::option_option)]
     #[serde(skip_serializing_if = "Option::is_none")]
     embeds: Option<Option<Vec<Embed>>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    payload_json: Option<Vec<u8>>,
 }
 
 /// Update a message created by a webhook.
@@ -148,6 +152,7 @@ struct UpdateWebhookMessageFields {
 /// [`DeleteWebhookMessage`]: super::DeleteWebhookMessage
 pub struct UpdateWebhookMessage<'a> {
     fields: UpdateWebhookMessageFields,
+    files: Vec<(String, Vec<u8>)>,
     fut: Option<Pending<'a, ()>>,
     http: &'a Client,
     message_id: MessageId,
@@ -171,6 +176,7 @@ impl<'a> UpdateWebhookMessage<'a> {
                 allowed_mentions: http.default_allowed_mentions(),
                 ..UpdateWebhookMessageFields::default()
             },
+            files: Vec::new(),
             fut: None,
             http,
             message_id,
@@ -187,6 +193,28 @@ impl<'a> UpdateWebhookMessage<'a> {
         self
     }
 
+    /// Specify an attachment already present in the target message to keep.
+    ///
+    /// If called, all unspecified attachments will be removed from the message.
+    /// If not called, all attachments will be kept.
+    pub fn attachment(mut self, attachment: Attachment) -> Self {
+        self.fields.attachments.push(attachment);
+
+        self
+    }
+
+    /// Specify multiple attachments already present in the target message to keep.
+    ///
+    /// If called, all unspecified attachments will be removed from the message.
+    /// If not called, all attachments will be kept.
+    pub fn attachments(mut self, attachments: impl IntoIterator<Item = Attachment>) -> Self {
+        self.fields
+            .attachments
+            .extend(attachments.into_iter().collect::<Vec<Attachment>>());
+
+        self
+    }
+
     /// Set the content of the message.
     ///
     /// Pass `None` if you want to remove the message content.
@@ -292,20 +320,81 @@ impl<'a> UpdateWebhookMessage<'a> {
         Ok(self)
     }
 
-    fn request(&self) -> Result<Request> {
-        let body = crate::json_to_vec(&self.fields).map_err(HttpError::json)?;
-        let route = Route::UpdateWebhookMessage {
-            message_id: self.message_id.0,
-            token: self.token.clone(),
-            webhook_id: self.webhook_id.0,
-        };
+    /// Attach a file to the webhook.
+    ///
+    /// This method is repeatable.
+    pub fn file(mut self, name: impl Into<String>, file: impl Into<Vec<u8>>) -> Self {
+        self.files.push((name.into(), file.into()));
 
-        Ok(if let Some(reason) = &self.reason {
-            let headers = request::audit_header(&reason)?;
-            Request::from((body, headers, route))
+        self
+    }
+
+    /// Attach multiple files to the webhook.
+    pub fn files<N: Into<String>, F: Into<Vec<u8>>>(
+        mut self,
+        attachments: impl IntoIterator<Item = (N, F)>,
+    ) -> Self {
+        for (name, file) in attachments {
+            self = self.file(name, file);
+        }
+
+        self
+    }
+
+    /// JSON encoded body of any additional request fields.
+    ///
+    /// If this method is called, all other fields are ignored, except for
+    /// [`file`]. See [Discord Docs/Create Message] and
+    /// [`ExecuteWebhook::payload_json`].
+    ///
+    /// [`file`]: Self::file
+    /// [`ExecuteWebhook::payload_json`]: super::ExecuteWebhook::payload_json
+    /// [Discord Docs/Create Message]: https://discord.com/developers/docs/resources/channel#create-message-params
+    pub fn payload_json(mut self, payload_json: impl Into<Vec<u8>>) -> Self {
+        self.fields.payload_json.replace(payload_json.into());
+
+        self
+    }
+
+    fn request(&mut self) -> Result<Request> {
+        if self.files.is_empty() {
+            let body = crate::json_to_vec(&self.fields).map_err(HttpError::json)?;
+
+            let route = Route::UpdateWebhookMessage {
+                message_id: self.message_id.0,
+                token: self.token.clone(),
+                webhook_id: self.webhook_id.0,
+            };
+
+            if let Some(reason) = &self.reason {
+                let headers = request::audit_header(&reason)?;
+                Ok(Request::from((body, headers, route)))
+            } else {
+                Ok(Request::from((body, route)))
+            }
         } else {
-            Request::from((body, route))
-        })
+            let mut multipart = Form::new();
+
+            for (index, (name, file)) in self.files.drain(..).enumerate() {
+                multipart.file(format!("{}", index).as_bytes(), name.as_bytes(), &file);
+            }
+
+            let body = crate::json_to_vec(&self.fields).map_err(HttpError::json)?;
+            multipart.part(b"payload_json", &body);
+
+            let route = Route::UpdateWebhookMessage {
+                message_id: self.message_id.0,
+                token: self.token.clone(),
+                webhook_id: self.webhook_id.0,
+            };
+
+            if let Some(reason) = &self.reason {
+                let headers = request::audit_header(&reason)?;
+                Ok(Request::from((multipart, headers, route)))
+            } else {
+                Ok(Request::from((multipart, route)))
+            }
+        }
     }
 
     fn start(&mut self) -> Result<()> {
@@ -340,7 +429,7 @@ mod tests {
     #[test]
     fn test_request() {
         let client = Client::new("token");
-        let builder = UpdateWebhookMessage::new(&client, WebhookId(1), "token", MessageId(2))
+        let mut builder = UpdateWebhookMessage::new(&client, WebhookId(1), "token", MessageId(2))
             .content(Some("test".to_owned()))
             .expect("'test' content couldn't be set")
             .reason("reason")
@@ -349,8 +438,10 @@ mod tests {
 
         let body = crate::json_to_vec(&UpdateWebhookMessageFields {
             allowed_mentions: None,
+            attachments: Vec::new(),
             content: Some(Some("test".to_owned())),
             embeds: None,
+            payload_json: None,
         })
         .expect("failed to serialize fields");
         let route = Route::UpdateWebhookMessage {
